@@ -25,6 +25,7 @@ except ImportError:  # pragma: NO COVER
 
 import google.cloud._helpers
 from google.cloud.bigquery import table
+from google.cloud.bigquery._pandas_helpers import _BIGNUMERIC_SUPPORT
 from google.cloud.bigquery.dbapi import _helpers
 from google.cloud.bigquery.dbapi import exceptions
 from tests.unit.helpers import _to_pyarrow
@@ -51,6 +52,14 @@ class TestQueryParameters(unittest.TestCase):
                 "TIMESTAMP",
             ),
         ]
+        if _BIGNUMERIC_SUPPORT:
+            expected_types.append(
+                (
+                    decimal.Decimal("1.1234567890123456789012345678901234567890"),
+                    "BIGNUMERIC",
+                )
+            )
+
         for value, expected_type in expected_types:
             msg = "value: {} expected_type: {}".format(value, expected_type)
             parameter = _helpers.scalar_to_query_parameter(value)
@@ -104,6 +113,11 @@ class TestQueryParameters(unittest.TestCase):
             ),
         ]
 
+        if _BIGNUMERIC_SUPPORT:
+            expected_types.append(
+                ([decimal.Decimal("{d38}.{d38}".format(d38="9" * 38))], "BIGNUMERIC")
+            )
+
         for values, expected_type in expected_types:
             msg = "value: {} expected_type: {}".format(values, expected_type)
             parameter = _helpers.array_to_query_parameter(values)
